@@ -1,10 +1,12 @@
 from actionpack.action import Action
 from actionpack.procedure import Procedure
 from actionpack.procedure import KeyedProcedure
+from actionpack.action import partialaction
 
 
 __all__ = [
     'Action',
     'Procedure',
-    'KeyedProcedure'
+    'KeyedProcedure',
+    'partialaction'
 ]
