@@ -25,13 +25,11 @@ class Procedure(Generic[Name, Outcome]):
     def validate(self):
         actions, spare = tee(self.__actions, 2)
         self.__actions = spare
-        try:
-            for action in actions:
-                if not isinstance(action, Action):
-                    msg = f'Procedures can only execute Actions: {str(action)}'
-                    raise Procedure.NotAnAction(msg)
-        except Exception as e:
-            raise e
+        for action in actions:
+            if not isinstance(action, Action):
+                msg = f'Procedures can only execute Actions: {str(action)}'
+                raise Procedure.NotAnAction(msg)
+        return self
 
     def execute(
         self,
@@ -49,6 +47,7 @@ class Procedure(Generic[Name, Outcome]):
                 futures = {executor.submit(action._perform, should_raise=should_raise): str(action) for action in actions}
                 for future in as_completed(futures):
                     yield future.result()
+        return True
 
     def __repr__(self):
         actions, spare = tee(self.actions, 2)
@@ -60,11 +59,12 @@ class Procedure(Generic[Name, Outcome]):
             more_actions = True
         except StopIteration:
             more_actions = False
-        header = f'\nProcedure for performing the following Actions:\n'
+        header = '\nProcedure for performing the following Actions:\n'
         tab = '  '
         bullet = f'{tab}* '
         action_list = reduce(lambda a, b: str(a) + f'\n{bullet}' + str(b), some_actions)
-        return header + bullet + str(action_list) + f'\n{tab}...\n' if more_actions else ''
+        footer = f'\n{tab}...\n' if more_actions else ''
+        return header + bullet + str(action_list) + footer
 
     def __iter__(self):
         return self._actions
@@ -91,9 +91,13 @@ class KeyedProcedure(Procedure[Name, Outcome]):
         actions, spare = tee(self.__actions, 2)
         self.__actions = spare
         for action in actions:
+            if not isinstance(action, Action):
+                msg = f'Procedures can only execute Actions: {str(action)}'
+                raise Procedure.NotAnAction(msg)
             if action.name is None:
                 msg = f'All {self.__class__.__name__} Actions must have a name: {str(action)}'
                 raise KeyedProcedure.UnnamedAction(msg)
+        return self
 
     def execute(
         self,
